@@ -1,35 +1,78 @@
 from functools import reduce
+import os
+from io import StringIO
+import requests
 import csv
 import numpy
 
+from ._s3_client import _load_from_bucket
+from .request import request_url, web_url
+
 DELIMITER = '\t'
 ENCODING = 'ISO-8859-1'
+GLOSSARY_FOLDER = 'glossary/lookups'
 
 
-def _recfromcsv_mod(filename: str, **kwargs):
-    def rewrite_csv_as_tab():
-        with open(filename, 'r', encoding=ENCODING) as fp:
-            reader = csv.reader(fp)
-            for row in reader:
-                yield DELIMITER.join(row)
-    return numpy.recfromcsv(rewrite_csv_as_tab(), delimiter=DELIMITER, **kwargs)
+def _rewrite_csv_file_as_tab(filepath: str):
+    with open(filepath, 'r', encoding=ENCODING) as fp:
+        reader = csv.reader(fp)
+        for row in reader:
+            yield DELIMITER.join(row)
 
 
-def load_lookup(filename: str):
+def _rewrite_csv_text_as_tab(text: str):
+    reader = csv.reader(StringIO(text))
+    for row in reader:
+        yield DELIMITER.join(row)
+
+
+def _recfromcsv(data, **kwargs): return numpy.recfromcsv(data, delimiter=DELIMITER, **kwargs)
+
+
+def load_lookup(filepath: str):
     """
-    Import lookup table as csv file into a `numpy.recarray`.
+    Import local lookup table as csv file into a `numpy.recarray`.
 
     Parameters
     ----------
-    filename : str
-        The name of the file.
+    filepath : str
+        The path of csv file on the local file system.
 
     Returns
     -------
     numpy.recarray
         The `numpy.recarray` converted from the csv content.
     """
-    return _recfromcsv_mod(filename, encoding=ENCODING)
+    return _recfromcsv(_rewrite_csv_file_as_tab(filepath), encoding=ENCODING)
+
+
+def _download_lookup_data(filename: str):
+    filepath = f"{GLOSSARY_FOLDER}/{filename}"
+    try:
+        return _load_from_bucket(os.getenv('AWS_BUCKET_GLOSSARY'), filepath).decode('utf-8')
+    except ImportError:
+        url = request_url(f"{web_url()}/{filepath}")
+        return requests.get(url).content.decode('utf-8')
+
+
+def download_lookup(filename: str):
+    """
+    Download lookup table from Hestia as csv into a `numpy.recarray`.
+
+    Parameters
+    ----------
+    filename : str
+        The name on the file on the Hestia lookup repository.
+
+    Returns
+    -------
+    numpy.recarray
+        The `numpy.recarray` converted from the csv content.
+    """
+    try:
+        return _recfromcsv(_rewrite_csv_text_as_tab(_download_lookup_data(filename)), encoding=ENCODING)
+    except ValueError:
+        return None
 
 
 def column_name(key: str):
@@ -46,7 +89,7 @@ def column_name(key: str):
     str
         The column name that can be used in `get_table_value`.
     """
-    return key.replace(',', '').replace(' ', '_').lower()
+    return key.replace(',', '').replace(' ', '_').replace('.', '').lower()
 
 
 def _get_single_table_value(array: numpy.recarray, col_match, col_match_with, col_val):
@@ -76,7 +119,6 @@ def get_table_value(array: numpy.recarray, col_match, col_match_with, col_val):
         The value found or `None` if no match.
     """
     def reducer(x, values):
-        print(values)
         col = values[1]
         value = col_match_with[values[0]]
         return x[x[col] == value]
